#!/usr/bin/php
<?php
//
// DB Maintenance Subsystem
// Copyright (c) 2017-2018 Nagios Enterprises, LLC. All rights reserved.
//
define('SUBSYSTEM', 1);
require_once(dirname(__FILE__) . '/../html/includes/base.inc.php');

subsys_get_options();
dbmaint_subsys();

// If the trial has expired, we don't delete anyhing anymore. So their old data still is visible
if (is_trial_expired()) {
	log_debug(LOG_TYPE_SYSTEM, "dbmaint_subsys: Trial is expired. No longer removing stale data.");
	exit(0);
}

function dbmaint_subsys() {

	global $db;

	$fusiondb = 'fusion';

	// Nagios Cloud systems only (update database with latest data from Cloud API)
	get_nagios_cloud_stats();

	dbmaint_get_retention_policies($log_retention, $polled_retention);
    $debug_in_db = get_option('debug_in_db');

	try {

		// clean up users_servers mappings
		dbmaint_query_and_log('DELETE 
			FROM users_servers 
			WHERE user_id NOT IN (SELECT user_id FROM users) 
			OR server_id NOT IN (SELECT server_id FROM servers)', 'users_servers');

		// clean up stale commands
		$command_stale_time = time() - (24 * 60 * 60); // 1 day
		dbmaint_query_and_log("DELETE
			FROM commands
			WHERE UNIX_TIMESTAMP(submission_time) < {$command_stale_time}
			OR status_code = " . COMMAND_STATUS_COMPLETED, 'commands');

		// clean up dashboards, dashlets, dashlet_params
		dbmaint_query_and_log('DELETE
			FROM dashboards
			WHERE user_id NOT IN (SELECT user_id FROM users)', 'dashboards');
		dbmaint_query_and_log('DELETE
			FROM dashlets
			WHERE dashboard_id NOT IN (SELECT dashboard_id FROM dashboards)', 'dashlets');
		dbmaint_query_and_log('DELETE
			FROM dashlets_params
			WHERE dashlet_id NOT IN (SELECT dashlet_id FROM dashlets)', 'dashlets_params');

		// clean up views
		dbmaint_query_and_log('DELETE
			FROM views
			WHERE user_id NOT IN (SELECT user_id FROM users)', 'views');

		// clean up auth tokens
		$auth_token_stale_time = time() - (24 * 60 * 60); // 1 day
		dbmaint_query_and_log("DELETE
			FROM auth_tokens
			WHERE UNIX_TIMESTAMP(auth_valid_until) < {$auth_token_stale_time}", 'auth_tokens');

		// clean up user meta
		$obj_type = META_TYPE_USER;
		dbmaint_query_and_log("DELETE 
			FROM meta 
			WHERE obj_type = {$obj_type} 
			AND obj_id NOT IN (SELECT user_id FROM users)", 'meta');

		// clean log
		dbmaint_query_and_log('DELETE
			FROM log
			WHERE UNIX_TIMESTAMP(log_time) < ' . (time() - $log_retention), 'log');

		// remove debugging from log if keep debugging in database is unchecked
		if (!$debug_in_db)
			dbmaint_query_and_log('DELETE
				FROM log
				WHERE level >= ' . LOG_LEVEL_DEBUG, 'log');

		// TODO:
		// check to make sure that mapped users make sense
		// if the type is for polling, then make sure that user exists in the last polled info

		// remove any invalid polling_locks
		dbmaint_query_and_log('DELETE 
			FROM polling_lock 
			WHERE server_id NOT IN (SELECT server_id FROM servers)', 'polling_lock');

		// dont need sysstat for non-existent servers
		dbmaint_query_and_log('DELETE 
			FROM sysstat 
			WHERE server_id != 0 
			AND server_id NOT IN (SELECT server_id FROM servers)', 'sysstat');

		// clean up polled data
		dbmaint_query_and_log('DELETE 
			FROM polled_data 
			WHERE server_id NOT IN (SELECT server_id FROM servers)', 'polled_data');

		dbmaint_query_and_log('DELETE
			FROM polled_data 
			WHERE UNIX_TIMESTAMP(polled_time) < ' . (time() - $polled_retention), 'polled_data');

		dbmaint_query_and_log('DELETE
			FROM polled_averages
			WHERE server_id NOT IN 
				(SELECT server_id FROM servers)', 'polled_averages');

		dbmaint_query_and_log("DELETE 
			FROM polled_averages 
			WHERE poll_key NOT IN 
				(SELECT COLUMN_NAME FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = '{$fusiondb}' AND TABLE_NAME = 'polled_data') 
			AND poll_key NOT IN 
				(SELECT poll_key FROM polled_extras)", 'polled_averages');

		dbmaint_query_and_log('DELETE
			FROM polled_deltas
			WHERE server_id NOT IN 
				(SELECT server_id FROM servers)', 'polled_deltas');

		dbmaint_query_and_log("DELETE 
			FROM polled_deltas 
			WHERE poll_key NOT IN 
				(SELECT COLUMN_NAME FROM information_schema.COLUMNS WHERE TABLE_SCHEMA = '{$fusiondb}' AND TABLE_NAME = 'polled_data') 
			AND poll_key NOT IN 
				(SELECT poll_key FROM polled_extras)", 'polled_deltas');

		dbmaint_query_and_log('DELETE 
			FROM polled_extras 
			WHERE polled_data_id NOT IN 
				(SELECT polled_data_id FROM polled_data)', 'polled_extras');

		$cbargs = array();
		do_callbacks(CALLBACK_DBMAINT, $cbargs);

	} catch (Exception $e) {

		log_error(LOG_TYPE_SYSTEM, 'dbmaint_subsys() caught exception: ' . $e->getMessage());
		log_error(LOG_TYPE_SYSTEM, 'dbmaint_subsys() db error: ' . one_line_print_r($db->get_errors()));
	}


	set_sysstat_data('dbmaint_subsys', SUBSYS_OK);
}

// pass by ref the retention policies in seconds
function dbmaint_get_retention_policies(&$log, &$polled) {

	// IT WAS NEVER MEANT TO BE KEPT FOREVER

	$log = get_option('log_retention', DEFAULT_LOG_RETENTION);
	if (is_numeric($log) && $log > 0)
		$log *= 86400;
	else
		$log = 86400 * DEFAULT_LOG_RETENTION;

	$polled = get_option('polled_retention', DEFAULT_POLLED_RETENTION);
	if (is_numeric($polled) && $polled > 0)
		$polled *= 86400;
	else
		$polled = 86400 * DEFAULT_POLLED_RETENTION;

	log_debug(LOG_TYPE_SYSTEM, "Using log_retention: [$log], polled_retention: [$polled]");
}

// wrapper function to query and log meaningful output
function dbmaint_query_and_log($sql, $table_name, $bind_array = null) {

	global $db;
	$result = $db->exec_query($sql, $bind_array);

	if (is_numeric($result))
		log_debug(LOG_TYPE_SYSTEM, "dbmaint_query_and_log() deleted {$result} rows from {$table_name}");
	else
		log_error(LOG_TYPE_SYSTEM, "dbmaint_query_and_log() unable to gather query data: {$sql}");
}
