<?php

// Check if user is an admin
function is_admin($user_id=0)
{

    // Use logged in user's ID
    if ($user_id == 0 && @isset($_SESSION["user_id"])) {
        $user_id = $_SESSION["user_id"];
    }

    $level = get_user_attr('level', $user_id, 0);
    if ($level == 1) {
        return true;
    }

    return false;
}


function add_user($username, $password, $name, $email, $enabled, $level = USER_LEVEL_NONPRIV) {

    global $db;

    if (get_user_id($username))
        return false;

    $db->query('INSERT INTO users (username, password, name, email, enabled, level) VALUES (:username, :password, :name, :email, :enabled, :level)');
    $db->bind(':username', $username, PDO::PARAM_STR);
    $db->bind(':password', hash('sha256', $password), PDO::PARAM_STR);
    $db->bind(':name', $name, PDO::PARAM_STR);
    $db->bind(':email', $email, PDO::PARAM_STR);
    $db->bind(':enabled', $enabled, PDO::PARAM_INT);
    $db->bind(':level', $level, PDO::PARAM_INT);

    $user_id = 0;
    if ($db->exec())
        $user_id = $db->last_insert_id();

    if ($user_id > 0) {

        set_user_attr('backend_ticket', random_string(32), $user_id);
        set_user_attr('password_set_time', time(), $user_id);

        $cbargs = array(
            'user_id' => $user_id,
            'username' => $username,
            'email' => $email,
            'level' => $level,
            );

        do_callbacks(CALLBACK_USER_CREATED, $cbargs);

        return $user_id;
    }

    return false;
}


function delete_user($user_id) {

    global $db;

    // can't delete a non-numeric user id, or nagiosadmin (anything below that is invalid as well)
    if (!is_numeric($user_id) || $user_id <= 1)
        return false;

    // can't delete your own user, either
    if (!empty($_SESSION['user_id']) && $_SESSION['user_id'] == $user_id)
        return false;

    // shouldn't delete invalid users!
    if (!is_valid_user_id($user_id))
        return false;

    // check if we are cloud
    if (is_cloud_license()) {
        $username = get_username($user_id);
        if ($username == "nagiosadmin") {
            return false;
        }
    }

    // get rid of all the user meta
    delete_user_meta('', $user_id, $all_meta = true);

    $db->query('DELETE FROM users WHERE user_id = :user_id');
    $db->bind(':user_id', $user_id, PDO::PARAM_INT);

    return $db->exec();
}


/**
 * @param string $username
 * @param string $password
 * @param string[ref] &$error - string to set messages with [messages to show users]
 * @param array[ref] &$debug - array to set messages with [detailed messages for backend use, defined in constants under CHECKCRED_FAILED]
 *
 * @return bool based on successful authentication
 */
// TODO: fix this. add logging
function check_login_credentials($username, $password, &$error, &$debug) {

    $default_error = _('Username/password do not match data stored in Nagios Fusion database.');

    // clear messages
    $error = '';
    $debug = array();

    // get the user id (potentially) and check for validity
    $user_id = get_user_id($username);
    if ($user_id === false) {
        $debug[] = CHECKCRED_FAILED_GET_USER_ID;
        $error = $default_error;
        return false;
    }

    // check if our account is locked
    if (get_user_attr('locked', $user_id) == 1) {
        $debug[] = CHECKCRED_FAILED_LOCKED;
        $error = _('This account is locked. Please contact your administrator.');
        return false;
    }

    // check if the account is enabled
    if (get_user_attr('enabled', $user_id) == 0) {
        $debug[] = CHECKCRED_FAILED_DISABLED;
        $error = _('This account is disabled. Please contact your administrator.');
        return false;
    }

    // try other authentication methods first
    $auth_ok = false;
    $cbargs = array(
        'credentials' => array(
            'username' => $username,
            'password' => $password,
        ),
        'login_ok' => 0,
        'debug' => array(),
        'error' => array(),
    );

    do_callbacks(CALLBACK_PROCESS_AUTH_INFO, $cbargs);

    $debug = array_merge($debug, $cbargs['debug']);
    $error = $cbargs['error'];

    // check the callback args to see if any component has changed them
    if ($cbargs['login_ok'] == 1) {
        $auth_ok = true;
        //send_to_audit_log(_("User authenticated against external credentials"), AUDITLOGTYPE_SECURITY, AUDITLOGSOURCE_NAGIOSXI, $username);
        return true;
    } else {
        $debug[] = CHECKCRED_FAILED_EXTAUTH;
    }

    // check if this user is set to skip_local auth
    $skip_local = (bool) get_user_attr('skip_local', $user_id);

    // always allow nagiosadmin to login via local
    if ($skip_local && /*$user_id != 1 &&*/ $username != 'nagiosadmin') {
        $debug[] = CHECKCRED_FAILED_SKIPLOCAL;
        $error = $default_error;
        return false;
    }

    // check to make sure the password matches
    $pwhash = get_user_attr('password', $user_id);
    if ($pwhash == hash('sha256', $password)) {
        log_info(LOG_TYPE_SECURITY, _('User authenticated against Nagios Fusion local credentials'), 0, $user_id);
        return true;
    } else if ($pwhash == md5($password)) {
        // Removing this later but validate against md5 passwords too so Bryan's Fusion doesn't break :) may also be
        // useful if we write some sort of script to import old users into Fusion 4
        log_info(LOG_TYPE_SECURITY, _('User authenticated against deprecated Nagios Fusion local credentials'), 0, $user_id);
        return true;
    } else {
        $debug[] = CHECKCRED_FAILED_INVALIDPASS;
        $error = $default_error;
    }

    return false;
}

/**
 * get_user_id() - return a user_id of a specific username
 * @param $username - the user to search for
 * @return mixed false if not found , the user_id (int) otherwise
 */
function get_user_id($username) {

    global $db;

    if (empty($username))
        return false;

    $db->query('SELECT user_id FROM users WHERE username = :username LIMIT 1');
    $db->bind(':username', $username, PDO::PARAM_STR);
    $db->exec();

    $rows = $db->fetch_rows();
    if (count($rows) == 1) {

        $value = $rows[0]['user_id'];
        return intval($value);
    }

    return false;
}


/**
 * get_username() - return a username of a specific user_id
 * @param $userid - the userid to search for
 * @return mixed (bool)false if not found [or invalid data], the (string)username otherwise
 */
function get_username($user_id) {

    global $db;

    if (!is_numeric($user_id))
        return false;

    $db->query('SELECT username FROM users WHERE user_id = :user_id LIMIT 1');
    $db->bind(':user_id', $user_id);
    $db->exec();

    $rows = $db->fetch_rows();
    if (count($rows) == 1) {

        $value = $rows[0]['username'];
        return $value;
    }

    return false;
}


/**
 * get_user_meta_session_vars() - set session variables for currently logged in user
 * @param bool $overwrite - overwrite current session vars if true
 */
function get_user_meta_session_vars($overwrite = false) {

    global $db;

    if (!isset($_SESSION['user_id']))
        return null;

    $obj_type = META_TYPE_USER;
    $db->query("SELECT * FROM meta WHERE obj_type = $obj_type AND obj_id = :user_id AND autoload = 1");
    $db->bind(':user_id', intval($_SESSION['user_id']));
    $db->exec();

    $rows = $db->fetch_rows();
    if (count($rows) > 0) {

        foreach ($rows as $row) {
            foreach ($row as $key => $val) {

                if ($key == 'user_id')
                    continue;

                if ($overwrite || !isset($_SESSION[$key]))
                    $_SESSION[$key] = $val;
            }
        }
    }
}


/**
 * get_user_meta() - pull a value from usermeta (alias to meta)
 * @param string $keyname - the key of the usermeta to pull the value for
 * @param mixed $default - if the query happens and doesn't get a result the function will return this value
 * @param int $user_id - if 0, set to current user (the user to pull the value for)
 * @return mixed false if problem with user_id is detected, the value if everything is successful, and $default if nothing found
 */
function get_user_meta($keyname, $default = null, $user_id = 0) {

    global $db;

    if ($user_id == 0 && isset($_SESSION['user_id']))
        $user_id = $_SESSION['user_id'];

    if (!is_numeric($user_id) || $user_id == 0)
        return false;

    $obj_type = META_TYPE_USER;
    $db->query("SELECT meta_value FROM meta WHERE obj_type = $obj_type AND obj_id = :user_id AND meta_key = :keyname LIMIT 1");
    $db->bind(':user_id', $user_id);
    $db->bind(':keyname', $keyname);
    $db->exec();

    $rows = $db->fetch_rows();
    if (count($rows) == 1) {

        $value = $rows[0]['meta_value'];

        return $value;
    }

    return $default;
}


/**
 * set_user_meta() - set a value in usermeta (alias to meta)
 * @param string $keyname - the key of the usermeta to set
 * @param string $keyvalue - the value to set
 * @param bool $autoload - refers to whether this usermeta should be loaded and stored into session data on login (and when this function is called)
 * @param int $user_id - if 0, set to current user (the user to pull the value for)
 * @return bool false if problem with user_id is detected, or the return value from the sql query execution
 */
function set_user_meta($keyname, $keyvalue, $autoload = true, $user_id = 0) {

    global $db;

    if ($user_id == 0 && isset($_SESSION['user_id']))
        $user_id = $_SESSION['user_id'];

    if (!is_numeric($user_id) || $user_id == 0)
        return false;

    $meta_exists = get_user_meta($keyname, null, $user_id);
    $sql = '';

    $obj_type = META_TYPE_USER;
    if ($meta_exists === null) {
        $sql = "INSERT INTO meta (obj_type, obj_id, meta_key, meta_value, autoload) VALUES ($obj_type, :user_id, :keyname, :keyvalue, :autoload)";
    } else {
        $sql = "UPDATE meta SET meta_value = :keyvalue, autoload = :autoload WHERE obj_type = $obj_type AND obj_id = :user_id AND meta_key = :keyname LIMIT 1";
    }

    $db->query($sql);
    $db->bind(':user_id', $user_id);
    $db->bind(':keyname', $keyname);
    $db->bind(':keyvalue', $keyvalue);
    $db->bind(':autoload', intval($autoload));
    $exec = $db->exec();

    if ($exec && $autoload)
        $_SESSION[$keyname] = $keyvalue;

    return $exec;
}


function delete_user_meta($keyname, $user_id = 0, $all_user_meta = false) {

    global $db;

    if ($user_id == 0 && isset($_SESSION['user_id']))
        $user_id = $_SESSION['user_id'];

    if (!is_numeric($user_id) || $user_id == 0)
        return false
;
    // if we are deleting everything then we don't need to specify a keyname
    $and_keyname = $all_user_meta ? '' : ' AND meta_key = :keyname';

    $obj_type = META_TYPE_USER;
    $db->query("DELETE FROM meta WHERE obj_type = $obj_type AND obj_id = :user_id $and_keyname");
    $db->bind(':user_id', $user_id);
    if (!$all_user_meta)
        $db->bind(':keyname', $keyname);

    return $db->exec();
}


function is_valid_user($identifier) {

    $value = false;

    // check what kind of data we're working with and call the appropriate function
    if (is_numeric($identifier)) {

        $value = get_username($identifier);

    } else if (is_string($identifier)) {

        $value = get_user_id($identifier);
    }

    return !($value === false);
}


function is_valid_user_id($user_id) {

    return is_valid_user($user_id);
}


function get_users() {

    global $db;

    $db->query('SELECT * FROM users');
    $db->exec();

    return $db->fetch_rows();
}


function masquerade_as_user_id($user_id = -1) {

    // only admins can masquerade
    if (is_admin() == false)
        return;

    if (!is_valid_user_id($user_id))
        return;

    $original_user = $_SESSION['username'];

    global $user_attributes;

    $username = get_user_attr('username', $user_id);
    
    deinit_session();
    init_session();

    // set session variables
    $_SESSION['user_id'] = $user_id;
    $_SESSION['username'] = $username;

    // load user session variables (e.g. preferences)
    get_user_meta_session_vars(true);

    // get the new user attributes
    unset($user_attributes);
    get_user_attributes();

    // TODO: logging here
    // log it
    //send_to_audit_log("User '" . $original_user . "' masqueraded as user '" . $username . "'", AUDITLOGTYPE_SECURITY);
}


/**
 * is_valid_user_attr() - determine if a specified attr is valid
 * @return bool
 */
function is_valid_user_attr($attr) {

    return is_valid_column_of_table($attr, 'users');
}


/**
 *
 */
function get_user_attr($attr, $user_id = 0, $default = null) {

    global $user_attributes;

    if (isset($user_attributes[$attr]) && $user_id == 0) {

        return $user_attributes[$attr];

    } else {

        global $db;

        if (!is_valid_user_attr($attr))
            return null;

        if ($user_id == 0 && isset($_SESSION['user_id']))
            $user_id = $_SESSION['user_id'];

        if (!is_numeric($user_id) || $user_id == 0)
            return null;

        $db->query("SELECT {$attr} AS attr FROM users WHERE user_id = :user_id LIMIT 1");
        $db->bind(':user_id', $user_id, PDO::PARAM_INT);
        $db->exec();

        $rows = $db->fetch_rows();
        if (count($rows) == 1) {

            $value = $rows[0]['attr'];
            if (isset($_SESSION['user_id']) && $user_id == $_SESSION['user_id'])
                $user_attributes[$attr] = $value;

            return $value;
        }
    }

    return $default;
}


/**
 *
 */
function set_user_attr($attr, $value, $user_id = 0) {

    global $user_attributes;
    global $db;

    if (!is_valid_user_attr($attr))
        return null;

    // if user_id is set to 0 or (session id is set and specified user id is session id)
    if ($user_id == 0 || (isset($_SESSION['user_id']) && $user_id == $_SESSION['user_id']))
        $user_attributes[$attr] = $value;

    if ($user_id == 0 && isset($_SESSION['user_id']))
        $user_id = $_SESSION['user_id'];

    if (!is_numeric($user_id) || $user_id == 0)
        return null;

    $db->query("UPDATE users SET {$attr} = :value WHERE user_id = :user_id LIMIT 1");
    $db->bind(':value', $value);
    $db->bind(':user_id', $user_id, PDO::PARAM_INT);
    $db->exec();
}


// TODO: use this EVERYWHERE
/**
 * check_user_id() - helper to determine if a good user id was passed to a function that can be used
 * define good: the user_id isn't checked for validity in the database, but this prevents a non-admin user
 *   from specifying another user's id
 * if there is no session, we assume the user id is good to go
 * if user_id = 0 then try and use the sessions current user id
 * if user_id is nonzero, and we're an admin, its good to go
 * if user_id is nonzero and we aren't an admin, but it is our user_id, then we're good to go
 * @param int $user_id - the user id to check for
 * @return mixed int|bool - bool false on failure, the integer of the user id that is okay to use
 */
function check_user_id($user_id) {

    $user_id = intval($user_id);

    if (!isset($_SESSION['user_id']))
        return $user_id;

    if ($user_id == 0)
        return $_SESSION['user_id'];

    if ($user_id != 0 && is_admin())
        return $user_id;

    if ($user_id == $_SESSION['user_id'])
        return $user_id;

    return false;
}


// Get the user's number format for returning a formatted number
function get_user_number_format($user_id = 0) {

    $number_format = get_option('default_number_format', NF_2);
    $number_format = get_user_meta('number_format', $number_format, $user_id);

    return $number_format;
}


/**
 * get_admin_quantity() - Queries the database to get the administrator count. 
 * @return mixed int|bool - On successful query, returns integer count of administrators. On failure, returns boolean false.
 */
function get_admin_quantity() {
    global $db;
    $db->query("SELECT COUNT(user_id) FROM users WHERE level=1");
    $db->exec();
    $rows = $db->fetch_rows();

    return $rows[0]["COUNT(user_id)"];
}


function get_authlevels()
{
    $levels = array(
        USER_LEVEL_NONPRIV => _("User"),
        USER_LEVEL_ADMIN => _("Admin")
    );
    return $levels;
}


////////////////////////////////////////////////////////////////////////////////
// AUTHENTICATION TOKENS
//////////////////////////////////////////////////////////////////////////////// 


/**
 * Creates a single-use authentication token for login/rapid response
 *
 * @param   int     $user_id        User ID to create token for
 * @param   int     $valid_until    Unix timestamp for when token becomes invalid
 * @param   array   $restrictions   Pages that should be allowed to be viewed, empty by default (so no restriction)
 * @return  string                  Single-use auth token
 */
function user_generate_auth_token($user_id=0, $valid_until=0, $restrictions='')
{
    global $db;

    if (empty($user_id)) {
        $user_id = $_SESSION['user_id'];
    }

    // Generate token and set time limit for auth (5 mins by default)
    $auth_token = sha1(uniqid());
    if (empty($valid_until)) {
        $valid_until = time() + (5 * 60);
    }

    // Set restrictions if there are any
    if (!empty($restrictions) || is_array($restrictions)) {
        $restrictions = base64_encode(serialize($restrictions));
    }

    $db->query("INSERT INTO auth_tokens (auth_user_id, auth_token, auth_valid_until, auth_expires_at, auth_restrictions)
                VALUES (:user_id, :auth_token, FROM_UNIXTIME(:valid_until), null, :restrictions)");
    $db->bind(':user_id', $user_id);
    $db->bind(':auth_token', $auth_token);
    $db->bind(':valid_until', $valid_until);
    $db->bind(':restrictions', $restrictions);
    $db->exec();

    return $auth_token;
}


/**
 * Verifies that an auth token is valid and has not yet expired.
 *
 * @param   string  $auth_token     Auth token
 * @return  bool                    True if valid (non-expired) auth token
 */
function user_is_valid_auth_token($auth_token)
{
    global $db;

    // Get token from the DB to validate
    $sql = "SELECT *, UNIX_TIMESTAMP(auth_valid_until) as valid_until_ts FROM auth_tokens WHERE auth_token = :auth_token AND auth_used = '0'";
    $db->query($sql);
    $db->bind(':auth_token', $auth_token);
    $db->exec();
    $rows = $db->fetch_rows();

    // Check to see if the auth token is valid
    if (!empty($rows)) {
        $tk = $rows[0];
        if (time() > $tk['valid_until_ts']) {
            return false;
        }
        return true;
    }

    return false;
}


/**
 * Get the user ID from the auth token specified.
 *
 * @param   string  $auth_token     Auth token
 * @return  int                     User ID or false on failure
 */
function user_get_auth_token_user_id($auth_token)
{
    global $db;

    $sql = "SELECT auth_token_id, auth_user_id FROM auth_tokens WHERE auth_token = :auth_token";
    $db->query($sql);
    $db->bind(':auth_token', $auth_token);
    $db->exec();
    $rows = $db->fetch_rows();

    if (!empty($rows)) {
        return $rows[0]['auth_user_id'];
    }

    return false;
}


/**
 * Sets an auth token's session ID in the database. Also automatically sets
 * the auth_used value to 1.
 *
 * @param   string  $auth_token     Auth token
 * @return  bool                    True if successful
 */
function user_set_auth_token_used($auth_token)
{
    global $db;

    $db->query("UPDATE auth_tokens SET auth_used = '1' WHERE auth_token = :auth_token");
    $db->bind(':auth_token', $auth_token);
    $db->exec();

    return true;
}

/**
 * Returns the current number of admin accounts. Used for
 * making sure the last administrative account is never
 * deleted.
 *
 * @return int     Number of current admin accounts
*/
function get_number_of_admins() {

    global $db;

    $db->query("SELECT COUNT(*) as number_of_admins FROM users WHERE (level = 1 AND enabled = 1)");
    $result = $db->exec();
    $raw_data = $db->fetch_rows();
    $number_of_admins = $raw_data[0]['number_of_admins'];
    $number_of_admins = intval($number_of_admins);

    return $number_of_admins;

}